#include "labelex.h"

typedef struct appdata {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *label;

} appdata_s;


static void
win_delete_request_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

static void
win_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = data;
	/* Let window go to hide state. */
	elm_win_lower(ad->win);
}


static void
btn_default_cb(void *data,Evas_Object *obj, void *event_info)
{

	appdata_s* ad = data;
	elm_object_text_set(ad->label,"<color=#FF4500FF>Button Clicked</color>");

}

/*
 * This method is used to arrange the UI components in a specific order
 */
static void
object_pack(Evas_Object *box, Evas_Object *child, double horizontal_size,double vertical_size,double horizontal_position,double vertical_position)
{

	//This is a hint on how a container object should resize a given child within its area
	evas_object_size_hint_weight_set(child,horizontal_size,vertical_size);
	//Sets the hints for an object's alignment
	evas_object_size_hint_align_set(child,horizontal_position,vertical_position);
	//Makes the given Evas Object Visible
	evas_object_show(child);

	//Put the child to the end of the box
	elm_box_pack_end(box,child);

}

static void
create_base_gui(appdata_s *ad)
{
	/* Window */
	/* Create and initialize elm_win.
	   elm_win is mandatory to manipulate window. */

	ad->win = elm_win_util_standard_add(PACKAGE, PACKAGE);
	elm_win_autodel_set(ad->win, EINA_TRUE);

	if (elm_win_wm_rotation_supported_get(ad->win)) {
		int rots[4] = { 0, 90, 180, 270 };
		elm_win_wm_rotation_available_rotations_set(ad->win, (const int *)(&rots), 4);
	}

	evas_object_smart_callback_add(ad->win, "delete,request", win_delete_request_cb, NULL);
	eext_object_event_callback_add(ad->win, EEXT_CALLBACK_BACK, win_back_cb, ad);

	/* Conformant */
	/* Create and initialize elm_conformant.
	   elm_conformant is mandatory for base gui to have proper size
	   when indicator or virtual keypad is visible. */
	ad->conform = elm_conformant_add(ad->win);
	elm_win_indicator_mode_set(ad->win, ELM_WIN_INDICATOR_SHOW);
	elm_win_indicator_opacity_set(ad->win, ELM_WIN_INDICATOR_OPAQUE);
	evas_object_size_hint_weight_set(ad->conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(ad->win, ad->conform);
	evas_object_show(ad->conform);

	Evas_Object *box = elm_box_add(ad->win);
	evas_object_size_hint_weight_set(box,EVAS_HINT_EXPAND,EVAS_HINT_EXPAND);
	elm_object_content_set(ad->conform,box);
	evas_object_show(box);

	/* Label */

	ad->label = elm_label_add(ad->conform);
	elm_object_text_set(ad->label, _("<font_size=20>Hello EFL Object (Font Size 20) </font_size>"));
	object_pack(box,ad->label,0.0,1.0,0.5,0.5);


	/* Label */
	ad->label = elm_label_add(ad->conform);
	elm_object_text_set(ad->label,"<align=center>Press Button</align>");
	object_pack(box,ad->label,0.0,1.0,0.5,0.5);

	/* Button */

	Evas_Object * btn = elm_button_add(ad->conform);
	elm_object_text_set(btn,"Click Here");
	evas_object_smart_callback_add(btn,"clicked",btn_default_cb,ad);
	object_pack(box,btn,0.0,1.0,-1.0,1.0);

	/* Radio Button */

	Evas_Object *radio,*radio_group;
	radio = elm_radio_add(ad->conform);
	elm_object_text_set(radio,"Tizen Native");
	elm_radio_state_value_set(radio,1);
	radio_group = radio;
	object_pack(box,radio,1,1,-1,-1);


	radio = elm_radio_add(ad->conform);
	elm_object_text_set(radio,"Tizen Web");
	elm_radio_state_value_set(radio,2);
	elm_radio_group_add(radio,radio_group);
	object_pack(box,radio,1,1,-1,-1);

	/* Entry */

	Evas_Object *textInput = elm_entry_add(ad->conform);
	elm_entry_entry_set(textInput,"Enter Text .....");
	object_pack(box,textInput,1,1,-1,-1);


	/* Show window after base gui is set up */
	evas_object_show(ad->win);

}

static bool
app_create(void *data)
{
	/* Hook to take necessary actions before main event loop starts
		Initialize UI resources and application's data
		If this function returns true, the main loop of application starts
		If this function returns false, the application is terminated */
	appdata_s *ad = data;

	create_base_gui(ad);

	return true;
}

static void
app_control(app_control_h app_control, void *data)
{
	/* Handle the launch request. */
}

static void
app_pause(void *data)
{
	/* Take necessary actions when application becomes invisible. */
}

static void
app_resume(void *data)
{
	/* Take necessary actions when application becomes visible. */

}

static void
app_terminate(void *data)
{
	/* Release all resources. */
}

static void
ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;
	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);
	elm_language_set(locale);
	free(locale);
	return;
}

static void
ui_app_orient_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_DEVICE_ORIENTATION_CHANGED*/
	return;
}

static void
ui_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_REGION_FORMAT_CHANGED*/
}

static void
ui_app_low_battery(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_BATTERY*/
}

static void
ui_app_low_memory(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_MEMORY*/
}

int
main(int argc, char *argv[])
{
	appdata_s ad = {0,};
	int ret = 0;

	ui_app_lifecycle_callback_s event_callback = {0,};
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;

	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_BATTERY], APP_EVENT_LOW_BATTERY, ui_app_low_battery, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_MEMORY], APP_EVENT_LOW_MEMORY, ui_app_low_memory, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_DEVICE_ORIENTATION_CHANGED], APP_EVENT_DEVICE_ORIENTATION_CHANGED, ui_app_orient_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, ui_app_lang_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED], APP_EVENT_REGION_FORMAT_CHANGED, ui_app_region_changed, &ad);

	ret = ui_app_main(argc, argv, &event_callback, &ad);
	if (ret != APP_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "app_main() is failed. err = %d", ret);
	}

	return ret;
}
